from django.contrib import admin
from django.db import models

from .models import LinkItem


class LinkItemInlines(admin.StackedInline):
    model = LinkItem
    fk_name = "parent"
    verbose_name = "Child Link"
    verbose_name_plural = "Child Links"

    extra = 0

    fieldsets = (
        ("", {
            'fields': (
                ('order', 'display_name', 'url', ),
            )
        }),
    )
    ordering = ('order',)


class LinkItemAdmin(admin.ModelAdmin):

    fieldsets = (
        ('Link Items', {'fields': ('parent', 'display_name', 'txtid',
         ('hide'),
         'url')}),
    )

    list_display = ('admin_title', 'display_name',)
    list_display_links = ('display_name',)
    list_filter = ('parent',)
    readonly_fields = ('txtid',)

    autocomplete_lookup_fields = {
        'fk': ['parent']
    }
    raw_id_fields = ('parent',)
    inlines = [LinkItemInlines]
    ordering = ('path',)

    def reindex_items(self, request, queryset):
        for item in queryset:
            LinkItemIndex().update_object(item)
        messages.success(request, '%s items reindexed.' % (len(queryset)))

    def lookup_allowed(self, key, value):
        """Enable Admin Filtering by Parent TextID"""
        if key in ('parent__txtid',):
            return True

        return super(LinkItemAdmin, self).lookup_allowed(key, value)


admin.site.register(LinkItem, LinkItemAdmin)



#!/usr/bin/env python
from setuptools import setup, find_packages
setup(name='django_linksets',
      description='A basic model to include administrable linksets\
      in templates.',
      version='2.0.1',
      url='https://github.com/gethee2anunnery/django-linksets',
      author="Sara Perry w/ Nina Pavlich",
      author_email='paraserry@gmail.com',
      packages=find_packages(exclude=['ez_setup']),
      zip_safe=False,
      include_package_data=True,
      install_requires=[
          'setuptools',
          'Django',
      ],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Intended Audience :: Developers',
          'License :: OSI Approved',
          'Operating System :: OS Independent',
          'Programming Language :: Python'
      ]
      )



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from .utils import unique_slugify

from django.db import models


class LinkItem(models.Model):

    parent = models.ForeignKey('self', blank=True, null=True)
    display_name = models.CharField(_("Display Name"), max_length=255,
                                    blank=False)

    txtid = models.CharField(_('txtid'), max_length=255, unique=True)
    order = models.PositiveIntegerField('Order', null=True,
                                        help_text="Order in which this\
                                        link appears in the set.")

    path = models.CharField(_('path'), max_length=255)

    url = models.CharField(_("URL"), max_length=255, blank=True,
                           help_text="Enter the full path for an external\
                           link, i.e. http://www.website.com or relative \
                           path for internal link, i.e. /login/")
    hide = models.BooleanField(_("Hide from LinkSet"), default=False)

    def save(self, *args, **kwargs):

        self.path = self.generate_path()

        unique_slugify(self, self.display_name, 'txtid')

        # Dont let link item point to self
        if self.parent and self.parent == self:
            self.parent = None

        super(LinkItem, self).save(*args, **kwargs)

    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "display_name__icontains",
                "parent__display_name")

    @property
    def admin_levels(self):
        if self.parent:
            return "%s<span style='color:#fff'>|------</span>"
            % self.parent.admin_levels
        else:
            return ""

    @property
    def admin_title(self):
        if self.parent:
            return mark_safe("%s&lfloor; %s"
                             % (self.admin_levels, self.display_name))
        else:
            return self.display_name

    def generate_path(self):
        if self.parent:
            return "%s%s/%s/" % (self.parent.path, self.order, self.txtid)
        else:
            return "/%s/" % self.txtid

    class Meta:
        ordering = ['order']
        verbose_name = "Link Item"
        verbose_name_plural = "Link Items"

    def __unicode__(self):
        return self.display_name

    def get_children(self):
        return self.__class__.objects.filter(parent=self)\
            .exclude(hide=True).order_by('order')

    def get_path(self):
        if self.url:
            return self.url

    def get_target(self):
        if self.url:
            if 'http' in self.url:
                return 'target=_blank'
            else:
                return ''
        else:
            return ''



from django.template import Library

from ..models import LinkItem

register = Library()


@register.assignment_tag()
def get_link_descendants(txtid):
    """
    This tag gets all descendants of a link item.
    """

    output = []
    try:
        children = LinkItem.objects.filter(parent__txtid=txtid)\
            .exclude(hide=True).order_by('order')
        for child in children:
            output.append({
                'item': child,
                'children': get_link_descendants(child.txtid)
            })
    except:
        return None

    return output
